@@ -12,11 +12,12 @@
 
 #include <sbi/sbi_types.h>
 
-struct sbi_trap_regs;
-struct sbi_scratch;
+struct sbi_trap_context;
 
-int sbi_illegal_insn_handler(u32 hartid, ulong mcause, ulong insn,
-			     struct sbi_trap_regs *regs,
-			     struct sbi_scratch *scratch);
+typedef int (*illegal_insn_func)(ulong insn, struct sbi_trap_regs *regs);
+
+int truly_illegal_insn(ulong insn, struct sbi_trap_regs *regs);
+
+int sbi_illegal_insn_handler(struct sbi_trap_context *tcntx);
 
 #endif
