@@ -11,39 +11,13 @@
  *   Anup Patel <anup.patel@wdc.com>
  */
 
-#include <libfdt.h>
 #include <sbi/sbi_error.h>
-#include <sbi_utils/fdt/fdt_helper.h>
 #include <sbi_utils/i2c/fdt_i2c.h>
 
 /* List of FDT i2c adapter drivers generated at compile time */
-extern struct fdt_i2c_adapter *fdt_i2c_adapter_drivers[];
-extern unsigned long fdt_i2c_adapter_drivers_size;
+extern const struct fdt_driver *const fdt_i2c_adapter_drivers[];
 
-static int fdt_i2c_adapter_init(void *fdt, int nodeoff)
-{
-	int pos, rc;
-	struct fdt_i2c_adapter *drv;
-	const struct fdt_match *match;
-
-	/* Try all I2C drivers one-by-one */
-	for (pos = 0; pos < fdt_i2c_adapter_drivers_size; pos++) {
-		drv = fdt_i2c_adapter_drivers[pos];
-		match = fdt_match_node(fdt, nodeoff, drv->match_table);
-		if (match && drv->init) {
-			rc = drv->init(fdt, nodeoff, match);
-			if (rc == SBI_ENODEV)
-				continue;
-			if (rc)
-				return rc;
-			return 0;
-		}
-	}
-
-	return SBI_ENOSYS;
-}
-
-static int fdt_i2c_adapter_find(void *fdt, int nodeoff,
+static int fdt_i2c_adapter_find(const void *fdt, int nodeoff,
 				struct i2c_adapter **out_adapter)
 {
 	int rc;
@@ -51,7 +25,8 @@ static int fdt_i2c_adapter_find(void *fdt, int nodeoff,
 
 	if (!adapter) {
 		/* I2C adapter not found so initialize matching driver */
-		rc = fdt_i2c_adapter_init(fdt, nodeoff);
+		rc = fdt_driver_init_by_offset(fdt, nodeoff,
+					       fdt_i2c_adapter_drivers);
 		if (rc)
 			return rc;
 
@@ -67,7 +42,7 @@ static int fdt_i2c_adapter_find(void *fdt, int nodeoff,
 	return 0;
 }
 
-int fdt_i2c_adapter_get(void *fdt, int nodeoff,
+int fdt_i2c_adapter_get(const void *fdt, int nodeoff,
 			struct i2c_adapter **out_adapter)
 {
 	int rc;
