@@ -12,7 +12,7 @@
 #include <sbi_utils/serial/fdt_serial.h>
 #include <sbi_utils/serial/litex-uart.h>
 
-static int serial_litex_init(void *fdt, int nodeoff,
+static int serial_litex_init(const void *fdt, int nodeoff,
 			     const struct fdt_match *match)
 {
 	uint64_t reg_addr, reg_size;
@@ -33,7 +33,7 @@ static const struct fdt_match serial_litex_match[] = {
 	{ },
 };
 
-struct fdt_serial fdt_serial_litex = {
+const struct fdt_driver fdt_serial_litex = {
 	.match_table = serial_litex_match,
 	.init = serial_litex_init
 };
