@@ -11,7 +11,7 @@
 #include <sbi_utils/fdt/fdt_helper.h>
 #include <sbi_utils/fdt/fdt_fixup.h>
 
-static u64 sifive_fu540_tlbr_flush_limit(const struct fdt_match *match)
+static u64 sifive_fu540_tlbr_flush_limit(void)
 {
 	/*
 	 * The sfence.vma by virtual address does not work on
@@ -20,6 +20,13 @@ static u64 sifive_fu540_tlbr_flush_limit(const struct fdt_match *match)
 	return 0;
 }
 
+static int sifive_fu540_platform_init(const void *fdt, int nodeoff, const struct fdt_match *match)
+{
+	generic_platform_ops.get_tlbr_flush_limit = sifive_fu540_tlbr_flush_limit;
+
+	return 0;
+}
+
 static const struct fdt_match sifive_fu540_match[] = {
 	{ .compatible = "sifive,fu540" },
 	{ .compatible = "sifive,fu540g" },
@@ -28,7 +35,7 @@ static const struct fdt_match sifive_fu540_match[] = {
 	{ },
 };
 
-const struct platform_override sifive_fu540 = {
+const struct fdt_driver sifive_fu540 = {
 	.match_table = sifive_fu540_match,
-	.tlbr_flush_limit = sifive_fu540_tlbr_flush_limit,
+	.init = sifive_fu540_platform_init,
 };
