@@ -16,8 +16,8 @@
 #include <sbi/sbi_console.h>
 #include <sbi/sbi_timer.h>
 #include <sbi/riscv_io.h>
+#include <sbi_utils/fdt/fdt_driver.h>
 #include <sbi_utils/fdt/fdt_helper.h>
-#include <sbi_utils/reset/fdt_reset.h>
 #include <sbi_utils/i2c/fdt_i2c.h>
 
 struct pmic {
@@ -282,26 +282,27 @@ err:
 	return rc;
 }
 
-static int starfive_jh7110_final_init(bool cold_boot, void *fdt,
-				      const struct fdt_match *match)
+static int starfive_jh7110_final_init(bool cold_boot)
 {
 	if (cold_boot) {
+		const void *fdt = fdt_get_address();
+
 		fdt_driver_init_one(fdt, starfive_jh7110_reset_drivers);
 	}
 
-	return 0;
+	return generic_final_init(cold_boot);
 }
 
-static bool starfive_jh7110_cold_boot_allowed(u32 hartid,
-				   const struct fdt_match *match)
+static bool starfive_jh7110_cold_boot_allowed(u32 hartid)
 {
 	if (selected_hartid != -1)
 		return (selected_hartid == hartid);
 
-	return true;
+	return generic_cold_boot_allowed(hartid);
 }
 
-static void starfive_jh7110_fw_init(const void *fdt, const struct fdt_match *match)
+static int starfive_jh7110_platform_init(const void *fdt, int nodeoff,
+					 const struct fdt_match *match)
 {
 	const fdt32_t *val;
 	int len, coff;
@@ -312,6 +313,11 @@ static void starfive_jh7110_fw_init(const void *fdt, const struct fdt_match *mat
 		if (val && len >= sizeof(fdt32_t))
 			selected_hartid = (u32) fdt32_to_cpu(*val);
 	}
+
+	generic_platform_ops.cold_boot_allowed = starfive_jh7110_cold_boot_allowed;
+	generic_platform_ops.final_init = starfive_jh7110_final_init;
+
+	return 0;
 }
 
 static const struct fdt_match starfive_jh7110_match[] = {
@@ -319,9 +325,7 @@ static const struct fdt_match starfive_jh7110_match[] = {
 	{ },
 };
 
-const struct platform_override starfive_jh7110 = {
+const struct fdt_driver starfive_jh7110 = {
 	.match_table = starfive_jh7110_match,
-	.cold_boot_allowed = starfive_jh7110_cold_boot_allowed,
-	.fw_init = starfive_jh7110_fw_init,
-	.final_init = starfive_jh7110_final_init,
+	.init = starfive_jh7110_platform_init,
 };
